-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DamageTaken")

L:RegisterTranslations("enUS", function() return {
	["Damage taken"] = true,
	["DT"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage taken"] = "받은 피해량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local position = nil
local dirty = nil
local broadcastdirty = 0
local moduleName = "DamageTaken"

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 38177 $"):sub(12, -3)) or 1
mod.version = 4
mod.canMergeData = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = mod:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType( moduleName, "hit", mod.OnParserEvent)
	parser:RegisterInfoType( moduleName, "environment", mod.OnParserEvent)
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(moduleName)
end

function mod:OnReset()
	if type(data) == "table" then
		data = mod:del(data)
	end

	data = mod:new()
	data[playerName] = 0
	dirty = nil
	broadcastdirty = 0
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return mod.revision end
function mod:GetDisplayName() return L["Damage taken"] end
function mod:GetShortDisplayName() return L["DT"] end
function mod:GetBroadcastData()
	local x = broadcastdirty
	broadcastdirty = 0
	return x
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:IsBroadcastDirty()
	return broadcastdirty > 0 and true or false
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

function mod:OnParserEvent( info )
	local victim, amount = info.victim, info.amount

	if not victim or victim == ParserLib_NONE or not amount or amount == 0 then
		return
	end
	if victim == ParserLib_SELF then
		data[playerName] = data[playerName] + amount
		broadcastdirty = broadcastdirty + amount
		dirty = true
	elseif Violation:AcceptDataFromSource( victim, moduleName ) then
		mod:OnRemoteData( victim, amount )
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end


