-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Healing")

L:RegisterTranslations("enUS", function() return {
	["Healing done"] = true,
	["H"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing done"] = "치유량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local data = nil
local broadcastdirty = 0
local dirty = nil
local moduleName = "Healing"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName, "AceEvent-2.0")
mod.revision = tonumber(("$Revision: 38301 $"):sub(12, -3)) or 1
mod.version = 4
mod.canMergeData = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = mod:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType(moduleName, "heal", self.OnParserEvent)
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(moduleName)
end

function mod:OnReset()
	if type(data) == "table" then
		data = mod:del(data)
	end
	data = mod:new()
	data[playerName] = 0
	broadcastdirty = 0
	dirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Healing done"] end
function mod:GetShortDisplayName() return L["H"] end
function mod:GetBroadcastData()
	local x = broadcastdirty
	broadcastdirty = 0
	return x
end

function mod:IsBroadcastDirty()
	return broadcastdirty > 0 and true or false
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent(info)
	local source, amount, victim = info.source, info.amount, info.victim

	if not source or source == ParserLib_NONE or not amount or amount == 0 or not victim then
		return
	end

	if not Violation:AcceptDataFromSource( source, moduleName ) then
		return
	end

	local victimid
	if victim == ParserLib_SELF then
		victimid = "player"
	else
		victimid = roster:GetUnitIDFromName( victim )
	end

	amount = amount - getOverheal( victimid, amount )
	if amount > 0 then
		if source == ParserLib_SELF then
			data[playerName] = data[playerName] + amount
			broadcastdirty = broadcastdirty + amount
			dirty = true
		else 
			mod:OnRemoteData( source, amount )
		end
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end
