-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Banzai")

local Banzai = AceLibrary("Banzai-1.1")
local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local broadcastdirty = nil
local dirty = nil
local moduleName = "Banzai"
local gainedAggroTime

local PERIODICUPDATE_ID = "Violation-"..moduleName.."-PeriodicUpdate"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 38177 $"):sub(12, -3)) or 1
mod.version = 4

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = mod:new()
end

function mod:OnModuleEnable()
	self:RegisterEvent("Banzai_UnitGainedAggro", 
		function(id) 
			local unit = Roster:GetUnitObjectFromUnit(id)
			if unit then
				self:GainedAggro(unit.name) 
			end
		end
	)
	self:RegisterEvent("Banzai_UnitLostAggro",
		function(id) 
			local unit = Roster:GetUnitObjectFromUnit(id)
			if unit then
				self:LostAggro(unit.name)
			end
		end
	)
	
	self:SetupAggroTable()
end

function mod:OnModuleDisable()
	self:ClearAggroTable()
	self:CancelAllScheduledEvents()
end

function mod:OnReset()
	self:ClearAggroTable()

	if type(data) == "table" then
		data = mod:del(data)
	end

	data = mod:new()
	data[playerName] = 0
	
	self:SetupAggroTable()
	
	dirty = nil
	broadcastdirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return "Aggro duration" end
function mod:GetShortDisplayName() return "Aggro" end
function mod:GetBroadcastData()
	local data = broadcastdirty
	broadcastdirty = nil
	return data
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:IsBroadcastDirty()
	return broadcastdirty
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:ClearAggroTable()
	if gainedAggroTime then
		for name in pairs(gainedAggroTime) do
			self:LostAggro(name)
		end
		gainedAggroTime = mod:del(gainedAggroTime)
	end
end

function mod:SetupAggroTable()
	gainedAggroTime = mod:del(gainedAggroTime) -- we never know
	gainedAggroTime = mod:new()
	for unit in Roster:IterateRoster(true) do
		if Banzai:GetUnitAggroByUnitName(unit.name) then
			self:GainedAggro(unit.name)
		end
	end
end

local function updateData(self, name)
	if not gainedAggroTime[name] then return end
	local now = GetTime()
	local delta = now - gainedAggroTime[name]
	gainedAggroTime[name] = now
	if delta > 0 then
		if Violation:AcceptDataFromSource(name, moduleName) then
			self:OnRemoteData(name, delta)
			if name == playerName then
				broadcastdirty = (broadcastdirty or 0) + delta
			end
		end
	end	
end

function mod:LostAggro(name)
	if gainedAggroTime[name] then
		updateData(self, name)
		gainedAggroTime[name] = nil
	end
end

function mod:GainedAggro(name)
	if not gainedAggroTime[name] then
		gainedAggroTime[name] = GetTime()
	end
	if next(gainedAggroTime) and not self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:ScheduleRepeatingEvent(PERIODICUPDATE_ID, self.PeriodicUpdate, 1, self) 
	end
end

function mod:PeriodicUpdate()
	for name in pairs(gainedAggroTime) do		
		updateData(self, name)
	end
	if not next(gainedAggroTime) and self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:CancelScheduledEvent(PERIODICUPDATE_ID) 
	end
end

local floor = math.floor
function mod.dataFormatter(duration)
	if duration >= 3600 then
		return ("%d:%02d:%02d"):format(floor(duration / 3600), floor(duration/60) % 60, floor(duration) % 60)
	else
		return ("%d:%02d"):format(floor(duration / 60), floor(duration) % 60)
	end
end